/// <reference types="node" />
declare const Mutex: any, Releaser: any;
declare const PromiseSocket: any;
declare enum BusOp {
    Nop = 0,
    Publish = 1,
    Subscribe = 2,
    Unsubscribe = 3,
    Message = 18,
    Broadcast = 19,
    Ack = 254
}
/** Bus call QoS */
export declare enum QoS {
    No = 0,
    Processed = 1,
    Realtime = 2,
    RealtimeProcessed = 3
}
/** Bus call errors (-32000 - code) */
export declare enum BusErrorCode {
    BusNotRegistered = -32113,
    Data = -32114,
    Io = -32115,
    Other = -32116,
    NotSupported = -32117,
    Busy = -32118,
    NotDelivered = -32119,
    Timeout = -32120,
    Access = -32121,
    RpcParse = -32700,
    RpcInvalidRequest = -32600,
    RpcMethodNotFound = -32601,
    RpcInvalidMethodParams = -32602,
    RpcInternal = -32603
}
declare enum RpcOp {
    Notification = 0,
    Request = 1,
    Reply = 17,
    Error = 18
}
/** @ignore */
interface FrameInterface {
    payload?: Buffer;
    header?: Buffer;
    op_topic?: Array<string> | string;
    qos: QoS;
    busOp: BusOp;
}
/** Bus frame */
export declare class Frame implements FrameInterface {
    /** @ignore */
    busOp: BusOp;
    /** @ignore */
    qos: number;
    /** @ignore */
    payload?: Buffer;
    /** @ignore */
    header?: Buffer;
    /** @ignore */
    payloadPos?: number;
    /** use this field to get the real frame sender */
    primary_sender?: string;
    /** frame sender with an optional secondary suffix */
    sender?: string;
    /** frame topic (in incoming publish events) */
    topic?: string;
    /** @ignore */
    op_topic?: string | Array<string>;
    /** @ignore */
    constructor(op: BusOp, qos?: number);
    /**
     * Gets frame payload
     *
     * @returns {Buffer}
     */
    getPayload(): Buffer;
}
/** Bus operation result */
export declare class OpResult {
    /** @ignore */
    qos: QoS;
    /** @ignore */
    locker: typeof Mutex;
    /** @ignore */
    result?: number;
    /** @ignore */
    release?: Promise<typeof Releaser>;
    /** @ignore */
    constructor(qos?: QoS);
    /**
     * Is operation completed
     *
     * @returns {boolean}
     */
    isCompleted(): boolean;
    /** @ignore */
    lock(): Promise<void>;
    /** @ignore */
    _waitCompletedCode(): Promise<number | undefined>;
    /**
     * Waits until the operation is completed
     *
     * @returns {Promise<void>}
     * @throws {BusError}
     */
    waitCompleted(): Promise<void>;
}
export declare class RpcOpResult {
    /** @ignore */
    completed: typeof Mutex;
    /** @ignore */
    frame?: Frame;
    /** @ignore */
    error?: BusError;
    /** @ignore */
    release?: Promise<typeof Releaser>;
    /** @ignore */
    constructor();
    /**
     * Is operation completed
     *
     * @returns {boolean}
     */
    isCompleted(): boolean;
    /** @ignore */
    lock(): Promise<void>;
    /**
     * Waits until the operation is completed
     *
     * @returns {Promise<RpcOpResult>}
     * @throws {BusError}
     */
    waitCompleted(): Promise<RpcOpResult>;
    /**
     * Gets frame payload
     *
     * @returns {Buffer}
     */
    getPayload(): Buffer;
}
/**
 * Incoming RPC-layer events
 */
export declare class RpcEvent {
    /** @ignore */
    op: RpcOp;
    /** bus frame */
    frame: Frame;
    /** @ignore */
    payloadPos: number;
    method?: Buffer;
    /** @ignore */
    callId?: number;
    /** @ignore */
    constructor(op: RpcOp, frame: Frame, payloadPos: number);
    /**
     * Gets frame payload
     *
     * @returns {Buffer}
     */
    getPayload(): Buffer | undefined;
}
/** Bus errors */
export declare class BusError {
    /** Error code */
    code: number;
    /** Error message */
    message?: Buffer;
    /**
     * @param {number} code - error code
     * @param {Buffer} [message] - encoded message payload
     */
    constructor(code: number, message?: Buffer);
}
/** Bus RPC layer */
export declare class Rpc {
    /** attached bus client */
    client: Bus;
    /** @ignore */
    callId: number;
    /** @ignore */
    call_lock: typeof Mutex;
    /** @ignore */
    calls: Map<number, RpcOpResult>;
    /** Method, called on incoming frames */
    onFrame?: (frame: RpcEvent) => Promise<void> | void;
    /** Method, called on incoming RPC notifications */
    onNotification?: (notification: RpcEvent) => Promise<void> | void;
    /** Method, called on incoming RPC calls */
    onCall: (call: RpcEvent) => Promise<Buffer | undefined> | Buffer | undefined;
    blockingNotifications: boolean;
    blockingFrames: boolean;
    /**
     * @param {Bus} client - Bus client
     */
    constructor(client: Bus);
    /**
     * Is RPC client connected
     *
     * @returns {boolean}
     */
    isConnected(): boolean;
    /**
     * Sends RPC notification
     *
     * @param {string} target - notification target
     * @param {Buffer} [payload] - payload
     * @param {QoS} [qos] - QoS
     *
     * @returns {Promise<OpResult>}
     */
    notify(target: string, payload?: Buffer, qos?: QoS): Promise<OpResult>;
    /**
     * Performs RPC call with no reply required
     *
     * @param {string} target - RPC target
     * @param {string} method - RPC method on the target
     * @param {Buffer} [payload] - payload
     * @param {QoS} [qos] - QoS
     *
     * @returns {Promise<OpResult>}
     */
    call0(target: string, method: string, params?: Buffer, qos?: QoS): Promise<OpResult>;
    /**
     * Performs RPC call
     *
     * @param {string} target - RPC target
     * @param {string} method - RPC method on the target
     * @param {Buffer} [payload] - payload
     * @param {QoS} [qos] - QoS
     *
     * @returns {Promise<RpcOpResult>}
     */
    call(target: string, method: string, params?: Buffer, qos?: QoS): Promise<RpcOpResult>;
    /** @ignore */
    _handleFrame(frame: Frame, me: Rpc): Promise<void>;
}
/** BUS/RT client */
export declare class Bus {
    /** assigned name */
    name: string;
    /** called on incoming frames */
    onFrame?: (frame: Frame) => void;
    /** called on disconnect */
    onDisconnect?: () => void;
    /** broker ping interval */
    pingInterval: number;
    /** @ignore */
    connected: boolean;
    /** bus timeout */
    timeout: number;
    /** @ignore */
    mgmt_lock: typeof Mutex;
    /** @ignore */
    socket_lock: typeof Mutex;
    /** @ignore */
    frameId: number;
    /** @ignore */
    frames: Map<number, OpResult>;
    /** @ignore */
    socket: typeof PromiseSocket;
    /**
     * @param {string} name - client name
     */
    constructor(name: string);
    /**
     * Connects the client
     *
     * @param {string} path - UNIX socket or TCP host:port
     *
     * @returns {Promise<void>}
     */
    connect(path: string): Promise<void>;
    /** @ignore */
    _tReader(me: Bus): Promise<void>;
    /** @ignore */
    _handleDaemonException(me: Bus, e: any): Promise<void>;
    /**
     * Sends a message to the target
     *
     * @param {string} target - bus target
     * @param {Buffer} [payload] - payload
     * @param {QoS} [qos] - QoS
     */
    send(target: string, payload?: Buffer, qos?: QoS): Promise<OpResult>;
    /** @ignore */
    _send(frame: FrameInterface, target?: string): Promise<OpResult>;
    /** @ignore */
    _tPing(me: Bus): Promise<void>;
    /**
     * Disconnects the bus client
     *
     * @returns {Promise<void>}
     */
    disconnect(): Promise<void>;
    /** @ignore */
    _disconnect(me: Bus): void;
    /**
     * Is bus client connected
     *
     * @returns {boolean}
     */
    isConnected(): boolean;
    /**
     * Subscribes client to topic(s)
     *
     * @param {string|Array<string>} topics - topics to subscribe
     *
     * @returns {Promise<OpResult>}
     */
    subscribe(topics: Array<string> | string): Promise<OpResult>;
    /**
     * Unsubscribes client from topic(s)
     *
     * @param {string|Array<string>} topics - topics to unsubscribe
     *
     * @returns {Promise<OpResult>}
     */
    unsubscribe(topics: Array<string>): Promise<OpResult>;
    /**
     * Publishes a message to a topic
     *
     * @param {string} topic - the target topic
     * @param {Buffer} [payload] - payload
     * @param {QoS} [qos] - QoS
     *
     * @returns {Promise<OpResult>}
     */
    publish(topic: string, payload?: Buffer, qos?: QoS): Promise<OpResult>;
}
export {};
